; Definitionen fr Mega16

; Belegungen der Atmel-Ports:

;*****************************************************************

;	  PA0		ADC0		Triggerbyte
;   PA1		ADC1
;	  PA2		ADC2
;   PA3		ADC3
;   PA4		ADC4
;   PA5		ADC5
;	  PA6		ADC6
;	  PA7		ADC7

;	  PB0		XCK/T0		Daten 0
;   PB1		T1			Daten 1
;   PB2		INT2/AIN0   Daten 2
;   PB3		OC0/AIN1    Daten 3
;   PB4		SS			Daten 4
;   PB5		MOSI		Daten 5
;	  PB6		MISO		Daten 6
;	  PB7		SCK			Daten 7

;   PC0		SCL			Triggermaske
;   PC1		SDA
;   PC2		TCK
;   PC3		IMS
;	  PC4		TDO
;	  PC5		TDI
;	  PC6		TOSC1
;	  PC7		TOSC2

;   PD0		RxD			RS232
;   PD1		TxD			RS232
;   PD2		INT0		Eingang externe Triggerauswertung, H Trigger
;   PD3		INT1		Umschaltung Sample/Lesen, H Lesen, L Sample
;   PD4	    OC1B 		Input-Buffer TriState H, aktiv L
;   PD5		OC1A		Clock fr Ram-Adresszhler
;   PD6		ICP1		Clock-Mux B
;   PD7		OC2			Clock-Mux A

;   Byte 1	Triggerbyte
;   Byte 2	Triggermaske
;   Byte 3	Timerintervall			(AVR Timerwert)
;   Byte 4	TimerScale				(AVR Prescaler)
;   Byte 5	Pretrigger	1...8, 12,5%, 25%, ... 87,5%, 0% (8)
;   Byte 6	ungenutzt
;   Byte 7	Mode


;Mode0: internal Clock, internal Trigger
;Mode1: frei
;Mode2: frei
;Mode3: frei
;Mode4: frei
;Mode5: Testpattern BinaryCounter
;Mode6: Testpattern 0xAA-0x55-0xAA-0x55...
;Mode7: Testpattern "Walking-Zero" (Period= 9 Clocks)

;	kompletten Ram nach dem Samplen kommentarlos zum PC schicken 

; 0xF0 wird fter als maximale Steuerbyte-Anzahl vom PC gesendet als Software-Reset
; 

;*****************************************************************

;--- Allgemeines -----------------------------------------------

.equ    F_CPU           = 16000000
.equ    CYCLES_PER_US   = ((F_CPU+500000)/1000000)

.equ    UART_BAUD_RATE  = 500000
;.equ    UART_BAUD_SELECT= (F_CPU/(UART_BAUD_RATE*16))-1
.equ    UART_BAUD_SELECT= (F_CPU/(UART_BAUD_RATE*8))-1			; U2X gesetzt

;--- Daten ------------------------------------------------

.equ	DATEN_DDR		= DDRB
.equ	DATEN_PORT	= PORTB
.equ	DATEN_IN		= PINB

.equ	CTRL_PORT		= PORTD
.equ	CTRL_PIN		= PIND

.equ	TRIG_IN			= PD2					; Eingang externe Triggerauswertung, H Trigger
.equ	C_SAMPLE		= PD3					; Umschaltung Sample/Lesen, H Lesen, L Sample
.equ	C_BUFFER		= PD4					; Input-Buffer TriState H, aktiv L
.equ	C_CLOCK			= PD5					; Takt fr Ram-Adresszhler (OC1A)
.equ	MUX_A			  = PD6
.equ	MUX_B			  = PD7

.equ	T_PORT			= PORTA

.equ	M_PORT			= PORTC

.equ	CLK_STOP		= (1<<WGM12)			; Timer Stop, CTC-Mode
.equ	CLK_START		= (1<<WGM12) | (1<<CS10); Timer Start, Prescale 1

.equ	MUX_INT			= 0b00000000
.equ	MUX_20MHZ		= 0b01000000
.equ	MUX_40MHZ		= 0b10000000
.equ	MUX_80MHz		= 0b11000000

.equ	MUX_MASKE		= 0b00111111

.equ	CMD_RESTART		= 0xF0
.equ	CMD_VERSION		= 0xF1
.equ	CMD_SIZE  		= 0xF2

.equ	CLK_80M			= 0		; ->  12,5ns
.equ	CLK_40M			= 0		; ->  25ns
.equ	CLK_20M			= 0		; ->  50ns
.equ	CLK_8M			= 0		; -> 125ns
.equ	CLK_4M			= 1		; -> 250ns
.equ	CLK_2M			= 3		; -> 500ns
.equ	CLK_800k		= 9 	; -> 1,25s
.equ	CLK_400k		= 19 	; -> 1,25s
.equ	CLK_200k		= 39	; -> 5s
.equ	CLK_80k 		= 99	; -> 12,5s
.equ	CLK_40k			= 199	; -> 20s
.equ	CLK_20k			= 399	; -> 50s
.equ	CLK_8k			= 999	; -> 125s
.equ	CLK_4k			= 1999	; -> 200s
.equ	CLK_2k			= 3999	; -> 500s
.equ	CLK_800			= 9999	; -> 1,25ms

.equ  RAM_SIZE    = 0x04

.equ	MODE_MAX		= 12										; maximale Anzahl Funktionen in der Tabelle

;**************************** Ram-Reservierungen ******************

;--- Internes Ram ------------------------------------------------

.DSEG

.ORG 0x100

BUF_LEN_L:	.byte			1			; Samplebufferlnge
BUF_LEN_H:	.byte 		1

.;**************************** Register-Bits **************************

;--- lower Register -----------------------------------------------

.def  MUL_L         = r0
.def  MUL_H         = r1

.def  BUF_LEN_REG   = r2

.def	NULL					= r3

;--- higher Register ----------------------------------------------

.def	PRE_TRIGGER		= r16
.def	MODE					= r17

.def	TRIG_BYTE			= r18

.def	SREG_SAVE			= r19

.def	TIMER_L				= r20
.def	TIMER_H				= r21

.def	TRIG_MASK			= r22
.def	TIMER_WERT		= r23

.def	TEMP_1				= r24
.def	TEMP_0				= r25

; r26/r27, r28/r29, r30/r31 fr X,Y,Z reserviert 

;--- Macros -------------------------------------------------------

.macro  load_p                      ; ldt 16Bit @1 nach @0 High und @0 Low

        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)

.endmacro

		
