
;**************************** Allgemeine Routinen **************************

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 ber UART
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************
uart_send_byte:
		sbis  UCSRA,UDRE                 ; Send-Register leer ?
    rjmp  uart_send_byte             ; nein

    out   UDR,TEMP_0
    ret

;******************************* Sub-UART **********************************
; Sub: Empfngt Byte ber UART
; Parameter:    -
; Return:       TEMP_0 Datenbyte
; Scratch-Reg:  -
;***************************************************************************

uart_receive_byte:
		sbis	UCSRA,RXC	
		rjmp 	uart_receive_byte				; nein

    in		TEMP_0,UDR							; holen


uart_receive_byte_end:
		ret

;*********************************************************************
; Sub: Send den Buffer-Inhalt
; Parameter:    Z zeigt auf erste Bufferposition
; Return:       -
; Scratch-Reg:  TEMP_0, Y, Z
;*********************************************************************

send_daten:
		lds			YL,BUF_LEN_L						; alle Daten senden
		lds			YH,BUF_LEN_H

send_daten_loop_1:
		in		TEMP_0,DATEN_IN					  ; Byte holen		

		rcall	uart_send_byte	
		sbi		CTRL_PORT,C_CLOCK				  ; Adresse weiterschalten
    nop
		cbi		CTRL_PORT,C_CLOCK
		sbiw	Y,1	
		brne	send_daten_loop_1

		ret

;*********************************************************************
; Sub: Gibt den REC-Interrupt frei fr Break
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0
;*********************************************************************

enable_break:
		sbis	UCSRA,RXC				      	; noch Daten zu holen?
		rjmp 	enable_break_ok					; nein

   	in		TEMP_0,UDR						  ; holen
		rjmp	enable_break

enable_break_ok:
		sei														; Interrupt fr Braek freigeben	

		ret

;*********************************************************************
; Sub: Lscht den Buffer
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0, Y
;*********************************************************************

clear_buffer:
		lds		YL,BUF_LEN_L
		lds		YH,BUF_LEN_H
	    sbi		CTRL_PORT,C_SAMPLE			; Mode Sample

		ser   TEMP_0                  ; alles Ausgang
		out		DATEN_DDR,TEMP_0		  	; und raus

clear_buffer_loop:
		out		DATEN_PORT,TEMP_0		  	; und raus

		sbi		CTRL_PORT,C_CLOCK
    nop
		cbi		CTRL_PORT,C_CLOCK
		sbiw	Y,1
		brne	clear_buffer_loop

		cbi		CTRL_PORT,C_CLOCK
		out		TCCR1A,NULL

		clr   TEMP_0
		out		DATEN_DDR,TEMP_0

		cbi		CTRL_PORT,C_SAMPLE				; Mode Lesen

		ret

;*********************************************************************
; Sub: Setzt Timing und Pre-Trigger Position und gibt Break frei
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0, Z, Y
;*********************************************************************

set_timing:
		rcall	clear_buffer

		load_p	Z,(2*tab_timer_intervall)

set_timing_loop:
		mov   TEMP_0,TIMER_WERT
    ldi   TEMP_1,6                ; Lnge eines Eintrags
    mul   TEMP_0,TEMP_1
    add   ZL,MUL_L
    adc   ZH,MUL_H           

		lpm		TIMER_L,Z+							; Timerwert holen
		lpm		TIMER_H,Z+

		lpm		YL,Z+										; Wert fr vollen Ram holen
		lpm		YH,Z+
		
		mov		XL,YL										; nach X kopieren fr Division / 8 (Pretrigger)
		mov		XH,YH

		lsr		XH
		ror		XL
		lsr		XH
		ror		XL
		lsr		XH
		ror		XL											; /8, 8 Abschnitte fr Pretrigger 

		mov		TEMP_0,PRE_TRIGGER

set_timer_trig_loop:
		cpi		TEMP_0,8								; = 100%
		breq	set_timer_trig_ok				; ja, ok

		inc		TEMP_0									; erhhen bis 8
		sub		YL,XL										; 1/8 abziehen
		sbc		YH,XH
		rjmp	set_timer_trig_loop

set_timer_trig_ok:						
		lpm		XL,Z+										; Wartezeit holen
		lpm		XH,Z

		out		T_PORT,TRIG_BYTE

		out		M_PORT,TRIG_MASK
	
		cpi		TIMER_WERT,3						; 20MHz oder grer?
		brlo	set_timer_ok						; ja, externer Quarztakt

set_timer_ctc:
		in		TEMP_0,CTRL_PORT				; MUX auf intern setzen
		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_INT
		out		CTRL_PORT,TEMP_0

		ldi		TEMP_0,(1<<COM1A0)			; 16Bit-Timer auf Toggle OC1A
		out		TCCR1A,TEMP_0

		out		OCR1AH,TIMER_H					; Timerwert raus
		out		OCR1AL,TIMER_L

		subi	TIMER_L,1	
		sbci	TIMER_H,0

		out		TCNT1H,TIMER_H					; Counter-Register auf Timerwert -1 damit beim ersten Takt getoggelt wird
		out		TCNT1L,TIMER_L

		ldi		TEMP_0,CLK_STOP					; CTC mit OCR1A, Prescale bleibt auf Stop
		out		TCCR1B,TEMP_0

set_timer_ok:									
		rcall	enable_break						; RX-Interrupt zum Abbruch
    				
		ret

;*********************************************************************
; Liest Daten, interner Takt, alle Triggervarianten
; Parameter:    -
; Return:       -
; Scratch-Reg:  TEMP_0, Z, Y, X
;*********************************************************************

int_clock_sample:

		rcall	set_timing
 	
		clt														    ; T als Trigger-Flag, erstmal lschen

		cbi		CTRL_PORT,C_BUFFER			    ; Eingang aktiv
		sbi		CTRL_PORT,C_SAMPLE			    ; Mode Sample

		cpi		TIMER_WERT,3						    ; 20/40/80MHz?
		brsh	int_clock_sample_int		    ; nein, max. 10MHz

		in		TEMP_0,CTRL_PORT
		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_20MHZ				   ; MUX auf 20MHz

		cpi		TIMER_WERT,2						   ; 20MHz?
		breq	int_clock_sample_20_80	   ; ja

		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_40MHz				   ; MUX auf 40MHz

		cpi		TIMER_WERT,1						   ; 40MHz?
		breq	int_clock_sample_20_80	   ; ja

		andi	TEMP_0,MUX_MASKE
		ori		TEMP_0,MUX_80MHz				   ; MUX auf 80MHz

int_clock_sample_20_80:
		out		CTRL_PORT,TEMP_0				   ; und Starten
    rjmp	int_clock_sample_loop

int_clock_sample_int:
		ldi		TEMP_1,CLK_START				    ; CTC mit OCR1A, Prescale 1
		out   TCCR1B,TEMP_1
   
int_clock_sample_loop:                ; einfach nur auf Trigger warten...
    sbic	CTRL_PIN,TRIG_IN            ; #2
		set														    ; #1 Trigger erkannt
		brts	int_clock_sample_wait		    ; #1 T wird auch vom Break-IRQ gesetzt, Y ist dann 1!

    rjmp  int_clock_sample_loop       ; #2  ein Durchlauf 6 Takte -> 300ns, bei 80NHz max. 30 Takte Abweichung

int_clock_sample_wait:
		sbiw	Y,1										    	; #2 Rest-Zhler
		breq	int_clock_sample_end	    	; #1   --	fertig

		mov		ZL,XL									    	; #1 Wartezeit setzen
		mov		ZH,XH									    	; #1

int_clock_sample_wait_loop:
		nop                               ; #1
    sbiw	Z,1											    ; #2 noch warten?
		breq	int_clock_sample_wait 	    ; #2 nein - #1 ja

		nop                               ;           #1 - 10
		nop                               ;           #1
		nop                               ;           #1
		nop                               ;           #1
 
    rjmp  int_clock_sample_wait_loop  ;           #2

int_clock_sample_end:
		cli

		in		TEMP_0,CTRL_PORT
		andi	TEMP_0,MUX_MASKE
		out		CTRL_PORT,TEMP_0

		ldi		TEMP_0,CLK_STOP				    	; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_0

		cbi		CTRL_PORT,C_CLOCK
		out		TCCR1A,NULL                 ; Clock Toggle vom Timer aus

		sbi		CTRL_PORT,C_BUFFER		    	; Eingang inaktiv
		cbi		CTRL_PORT,C_SAMPLE		    	; Mode Lesen

		ret

;*********************************************************************
; Sub: 0 Liest Daten, interner Takt, interner Trigger
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;*********************************************************************

int_clock_int_trigger:

		rcall	int_clock_sample
		
		ret 	

;*********************************************************************
; Sub: 2-4 Dummy-Funktion
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;*********************************************************************

dummy:

		ret

;*********************************************************************
; Sub: 5 Erzeugt Testpattern binary Counter
; Parameter:    -
; Return:       -
; Scratch-Reg:  Y, Z
;*********************************************************************

test_bin_count:
		ldi		TEMP_0,(1<<COM1A0)				; Toggle OC1A
		out		TCCR1A,TEMP_0

		ldi		TEMP_0,high(CLK_2M)				; Timing setzen
		out		OCR1AH,TEMP_0
		ldi		TEMP_0,low(CLK_2M)
		out		OCR1AL,TEMP_0

		ldi		TEMP_0,high(CLK_2M-1)			; TCNT auf 1 vor Compare setzen
		out		TCNT1H,TEMP_0							; damit mit einem Takt ohne Verzgerung gestartet wid
		ldi		TEMP_0,low(CLK_2M-1)			; begonnen wird
		out		TCNT1L,TEMP_0

		ldi		TEMP_0,CLK_STOP						; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_0

		lds		YL,BUF_LEN_L
		lds		YH,BUF_LEN_H
		sbi		CTRL_PORT,C_SAMPLE				; Mode Sample

		ser   TEMP_0
		out		DATEN_DDR,TEMP_0

		ldi		TEMP_1,CLK_START					; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_1

test_bin_count_loop:
		out		DATEN_PORT,YL   				  ; #1  und raus
		nop															; #1
		nop															; #1
		nop															; #1
		sbiw	Y,1												; #2
		brne	test_bin_count_loop				; #2	-> 8 Takte pro Loop = 2MHz

		ldi		TEMP_0,CLK_STOP						; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_0

		cbi		CTRL_PORT,C_CLOCK
		out		TCCR1A,NULL

		clr   TEMP_0
		out		DATEN_DDR,TEMP_0

		cbi		CTRL_PORT,C_SAMPLE				; Mode Lesen

		ldi		PRE_TRIGGER,8

		ret

;*********************************************************************
; Sub: 6 Erzeugt Testpattern 0x55/0xAA
; Parameter:    -
; Return:       -
; Scratch-Reg:  Y, Z
;*********************************************************************

test_pattern:

		ldi		TEMP_0,(1<<COM1A0)				; Toggle OC1A
		out		TCCR1A,TEMP_0

		ldi		TEMP_0,high(CLK_2M)				; Timing setzen
		out		OCR1AH,TEMP_0
		ldi		TEMP_0,low(CLK_2M)
		out		OCR1AL,TEMP_0

		ldi		TEMP_0,high(CLK_2M-1)			; TCNT auf 1 vor Compare setzen
		out		TCNT1H,TEMP_0							; damit mit einem Takt ohne Verzgerung
		ldi		TEMP_0,low(CLK_2M-1)			; begonnen wird
		out		TCNT1L,TEMP_0

		ldi		TEMP_0,CLK_STOP						; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_0

		lds		YL,BUF_LEN_L
		lds		YH,BUF_LEN_H
		sbi		CTRL_PORT,C_SAMPLE				; Mode Sample

		ser   TEMP_0
		out		DATEN_DDR,TEMP_0

		ldi		TEMP_0,0x55

		ldi		TEMP_1,CLK_START					; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_1

test_pattern_loop:
		out		DATEN_PORT,TEMP_0     		; #1  und raus

		com		TEMP_0    						  	; #1
		nop															; #1
		nop															; #1
		sbiw	Y,1												; #2
		brne	test_pattern_loop					; #2	-> 8 Takte pro Loop = 2MHz

		ldi		TEMP_0,CLK_STOP						; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_0	

		cbi		CTRL_PORT,C_CLOCK
		out		TCCR1A,NULL

		clr   TEMP_0
		out		DATEN_DDR,TEMP_0

		cbi		CTRL_PORT,C_SAMPLE				; Mode Lesen

		ldi		PRE_TRIGGER,8

		ret

;*********************************************************************
; Sub: 7 Erzeugt Testpattern walking zero
; Parameter:    -
; Return:       -
; Scratch-Reg:  Y, Z
;*********************************************************************

test_zero:

		ldi		TEMP_0,(1<<COM1A0)				; Toggle OC1A
		out		TCCR1A,TEMP_0

		ldi		TEMP_0,high(CLK_2M)				; Timing setzen
		out		OCR1AH,TEMP_0
		ldi		TEMP_0,low(CLK_2M)
		out		OCR1AL,TEMP_0

		ldi		TEMP_0,high(CLK_2M-1)			; TCNT auf 1 vor Compare setzen
		out		TCNT1H,TEMP_0							; damit mit einem Takt ohne Verzgerung
		ldi		TEMP_0,low(CLK_2M-1)			; begonnen wird
		out		TCNT1L,TEMP_0

		ldi		TEMP_0,CLK_STOP						; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_0

		lds		YL,BUF_LEN_L
		lds		YH,BUF_LEN_H
		sbi		CTRL_PORT,C_SAMPLE				; Mode Sample

    ser   TEMP_0
		out		DATEN_DDR,TEMP_0

		ldi		TEMP_0,0b11111110					; Bit 0 ist 0 zum durchschieben	

		ldi		TEMP_1,CLK_START					; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_1

test_zero_loop:
		out		DATEN_PORT,TEMP_0     		; #1

		bst		TEMP_0,7      						; #1 Bit 7 merken
		lsl		TEMP_0    	  						; #1
		bld		TEMP_0,0	      					; #1 ehemaliges Bit 7 in Bit 0 kopieren
		sbiw	Y,1												; #2
		brne	test_zero_loop						; #2	-> 8 Takte pro Loop = 2MHz

		ldi		TEMP_0,CLK_STOP						; CTC mit OCR1A, Prescale Stop
		out		TCCR1B,TEMP_0

		cbi		CTRL_PORT,C_CLOCK
		out		TCCR1A,NULL

    clr   TEMP_0
		out		DATEN_DDR,TEMP_0

		cbi		CTRL_PORT,C_SAMPLE				; Mode Lesen

		ldi		PRE_TRIGGER,8

		ret

;*********************************************************************
; Sub: sendet Version
; Parameter:    -
; Return:       -
; Scratch-Reg:  Y, Z
;*********************************************************************

send_version:

		load_p	Z, (2*text_version)

send_version_loop:
		lpm		TEMP_0,Z+
		tst		TEMP_0
		breq 	send_version_ram_size

		rcall	uart_send_byte
		rjmp	send_version_loop

send_version_ram_size:
		ldi		TEMP_0,RAM_SIZE
		swap	TEMP_0							; High-Nibble
		andi	TEMP_0,0x0F
		ori		TEMP_0,'0'
    cpi   TEMP_0,'0'
    brne  send_version_ok

    ldi   TEMP_0,' '

send_version_ok:
		rcall	uart_send_byte

		ldi		TEMP_0,RAM_SIZE
		andi	TEMP_0,0x0F					; Low-Nibble
		ori		TEMP_0,'0'
		rcall	uart_send_byte

		ldi		TEMP_0,'k'
		rcall	uart_send_byte
		ldi		TEMP_0,'B'
		rcall	uart_send_byte


send_version_end:
		ret

;*********************************************************************
; Sub: sendet eingestellte Buffergre
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;*********************************************************************

send_ram_size:

		load_p	Z, (2*text_ram)

send_ram_size_loop:
		lpm		TEMP_0,Z+
		tst		TEMP_0
		breq 	send_ram_size_end

		rcall	uart_send_byte
		rjmp	send_ram_size_loop

send_ram_size_end:
    ldi   TEMP_0,RAM_SIZE			    ; maximale Ramgre
    swap  TEMP_0
    andi  TEMP_0,0x0F
    ori   TEMP_0,'0'             
    cpi   TEMP_0,'0'
    brne  send_ram_size_ok

    ldi   TEMP_0,' '

send_ram_size_ok:
    rcall uart_send_byte

    ldi   TEMP_0,RAM_SIZE			    ; maximale Ramgre
    andi  TEMP_0,0x0F
    ori   TEMP_0,'0'             
    rcall uart_send_byte

    ldi   TEMP_0,'k'
    rcall uart_send_byte
    ldi   TEMP_0,'B'
    rcall uart_send_byte

		ret

;*********************************************************************
